@@ -5,14 +5,22 @@ import i18n
 
 
 class TimeDelta(object):
-    __slots__ = ("ago", "days", "hours", "minutes", "seconds")
+    __slots__ = ("ago", "days", "hours", "minutes", "seconds", "total_seconds")
 
-    def __init__(self, datetime_from, datetime_to):
-        delta = datetime_from - datetime_to
-        minutes, seconds = divmod(delta.days * 86400 + delta.seconds, 60)
+    def __init__(self, datetime_a, datetime_b):
+        if datetime_a > datetime_b:
+            self.ago = True
+        else:
+            self.ago = False
+            datetime_a, datetime_b = datetime_b, datetime_a
+
+        delta = datetime_a - datetime_b
+        total_seconds = delta.days * 86400 + delta.seconds
+        minutes, seconds = divmod(total_seconds, 60)
         hours, minutes = divmod(minutes, 60)
         days, hours = divmod(hours, 24)
-        self.days, self.hours, self.minutes, self.seconds = days, hours, minutes, seconds
+        self.total_seconds, self.days, self.hours, self.minutes, self.seconds = \
+            total_seconds, days, hours, minutes, seconds
 
     def __str__(self):
         return "Delta(%s day(s), %s hour(s), %s minute(s), %s second(s))" % (
@@ -22,7 +30,7 @@ class TimeDelta(object):
 _locales = {}
 
 
-class LocaleImportError(ImportError):
+class LocaleLoadError(ImportError):
     def __init__(self, locale, ex):
         self.message = "Cannot load locale module \"i18n.%s\". Exception: %s." % (locale, ex)
 
@@ -30,6 +38,14 @@ class LocaleImportError(ImportError):
         return self.message
 
 
+class InputArgumentError(StandardError):
+    def __init__(self, argument, ex):
+        self.message = "Invalid value for argument \"%s\". Exception: %s." % (argument, ex)
+
+    def __str__(self):
+        return self.message
+
+
 class LocaleContext(object):
     __slots__ = ("locale",)
 
@@ -47,17 +63,25 @@ class LocaleContext(object):
                     raise ValueError()
                 self.locale = locale
             except Exception, ex:
-                raise LocaleImportError(locale, ex)
+                raise LocaleLoadError(locale, ex)
 
-    def from_now(self, time, precision=1, from_utc=False):
-        if isinstance(time, datetime.datetime):
-            value = time
+    def from_now(self, value, precision=1, from_utc=False):
+        if isinstance(value, TimeDelta):
+            delta = value
         else:
-            value = (datetime.datetime.utcfromtimestamp if from_utc else datetime.datetime.fromtimestamp)(float(time))
+            if isinstance(value, datetime.datetime):
+                datetime_obj = value
+            else:
+                try:
+                    if from_utc:
+                        datetime_obj = datetime.datetime.utcfromtimestamp(value)
+                    else:
+                        datetime_obj = datetime.datetime.fromtimestamp(value)
+                except (TypeError, ValueError), ex:
+                    raise InputArgumentError("value", ex)
 
-        now = datetime.datetime.now()
-        ago = now > value
-        delta = TimeDelta(now, value) if ago else TimeDelta(value, now)
+            now = datetime.datetime.now()
+            delta = TimeDelta(now, datetime_obj)
 
         parts = []
         if precision and delta.days > 0:
@@ -72,7 +96,7 @@ class LocaleContext(object):
         if precision:
             parts.append(self.locale.relative(i18n.SECOND, delta.seconds))
 
-        return (self.locale.past if ago else self.locale.future)(", ".join(parts))
+        return (self.locale.past if delta.ago else self.locale.future)(", ".join(parts))
 
 
 _locale = LocaleContext()
