@@ -1,12 +1,16 @@
-SECOND = 0
-MINUTE = 1
-HOUR = 2
-DAY = 3
-MONTH = 4
-YEAR = 5
+YEAR = 0
+MONTH = 1
+DAY = 2
+HOUR = 3
+MINUTE = 4
+SECOND = 5
+MILLISECOND = 6
 
 
 class BaseLocale():
+    def __init__(self):
+        pass
+
     def relative(self, target, number):
         pass
 
